<?php

/**
 * Plugin Name: Foursquare Map
 * Plugin URI: http://digitalcortex.net/plugins/foursquare-map
 * Description: Adds a customisable map of your last Foursquare check-in location to a widget area. Makes it easy to share with your readers without giving away personal info. Totally user-defined.
 * Version: 0.1.3
 * Author: Tom Saunter
 * Author URI: http://digitalcortex.net/
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Add function to widgets_init that will load the widget.
 * @since 0.1
 */
add_action( 'widgets_init', 'four2map_load_widgets' );

/**
 * Register the widget.
 * 'Foursquare_Map_Widget' is the widget class used below.
 *
 * @since 0.1
 */
function four2map_load_widgets() {
	register_widget( 'Foursquare_Map_Widget' );
}

/**
 * Foursquare Map Widget class.
 * This class handles everything that needs to be handled with the widget:
 * the settings, form, display, and update. Nice!
 *
 * @since 0.1
 */
class Foursquare_Map_Widget extends WP_Widget {

	/**
	 * Widget setup.
	 */
	function Foursquare_Map_Widget() {
		/* Widget settings. */
		$widget_ops = array( 'classname' => 'four2map', 'description' => __('Adds your last Foursquare check-in location on a map', 'Foursquare Map') );

		/* Widget control settings. */
		$control_ops = array( 'width' => 260, 'height' => 350, 'id_base' => 'four2map-widget' );

		/* Create the widget. */
		$this->WP_Widget( 'four2map-widget', __('Foursquare Map Widget', 'four2map'), $widget_ops, $control_ops );
	}

	/**
	 * Display the widget on the screen.
	 */
	function widget( $args, $instance ) {
		extract( $args );
		
		/* Variables from the widget settings. */
		$title = apply_filters('widget_title', $instance['title'] );
		$fq = new fourSquare($instance['foursquare_username'], $instance['foursquare_password']);  //fetching the check-in data
		$map_size = $instance['map_size'];
		$map_zoom = $instance['map_zoom'];
		$show_comment = $instance['show_comment'];
		$label_venue = $instance['label_venue'];
		
		/* Before widget (defined by themes). */
		echo $before_widget;
		
		/* Display the widget title if one was input (before and after defined by themes). */
		if ( $title )
			echo $before_title . $title . $after_title;
		
		/* Displaying the Foursquare logo for the venue type. */
		if ($label_venue) {
			echo "<div class=".venue.">";
			//echo "<img class=".icon." src='".$fq->venueIcon."' />";
			echo "<span class=".label."><a href='http://foursquare.com/venue/".$fq->venueId."'>".$fq->venueName."</a></span>";
			echo "</div>";
		}
		
		/* Displaying the map. */
		echo "<a href='http://foursquare.com/user/-".$fq->userId."'><img class=".map." src=".$fq->getMapUrl($map_size, $map_size, $map_zoom, $fq->venueIcon)." /></a><br/>";
		
		/* Displaying the user comment. */
		if ($show_comment) {
			echo "<span class='".comment."'>".$fq->comment."</span><br />";
		}
		
		/* After widget (defined by themes). */
		echo $after_widget;
	}

	/**
	 * Update the widget settings.
	 */
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		/* Strip tags for title and name to remove HTML (important for text inputs) */
		$instance['title'] = strip_tags( $new_instance['title'] );
		$instance['foursquare_username'] = strip_tags( $new_instance['foursquare_username'] );
		$instance['foursquare_password'] = strip_tags( $new_instance['foursquare_password'] );
		$instance['map_size'] = strip_tags( $new_instance['map_size'] );
		$instance['map_zoom'] = strip_tags( $new_instance['map_zoom'] );
		$instance['show_comment'] = $new_instance['show_comment'];
		$instance['label_venue'] = $new_instance['label_venue'];

		return $instance;
	}

	/**
	 * Displays the widget settings controls on the widget panel.
	 * Makes use of the get_field_id() and get_field_name() function
	 * when creating your form elements. This handles the confusing stuff.
	 */
	function form( $instance ) {

		/* Setting up some default widget settings */
		$defaults = array(
			'title' => 'Last Check-in:',
			'foursquare_username' => 'your@address.com',
			'foursquare_password' => 'password',
			'map_size' => '250',
			'map_zoom' => '12',
			'show_comment' => true,
			'label_venue' => true,
		);
		$instance = wp_parse_args( (array) $instance, $defaults ); ?>

		<!-- Widget Title: Text Input -->
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e('Title:', 'four2map'); ?></label>
			<input id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $instance['title']; ?>" style="width:218px;" />
		</p>

		<!-- Username: Text Input -->
		<p>
			<label for="<?php echo $this->get_field_id( 'foursquare_username' ); ?>"><?php _e('Foursquare Email Address:', 'four2map'); ?></label>
			<input id="<?php echo $this->get_field_id( 'foursquare_username' ); ?>" name="<?php echo $this->get_field_name( 'foursquare_username' ); ?>" value="<?php echo $instance['foursquare_username']; ?>" style="width:253px;" />
		</p>
		
		<!-- Password: Text Input -->
		<p>
			<label for="<?php echo $this->get_field_id( 'foursquare_password' ); ?>"><?php _e('Foursquare Password:', 'four2map'); ?></label>
			<input type="password" id="<?php echo $this->get_field_id( 'foursquare_password' ); ?>" name="<?php echo $this->get_field_name( 'foursquare_password' ); ?>" value="<?php echo $instance['foursquare_password']; ?>" style="width:263px;" />
		</p>
		
		<!-- Map Size: Text Input -->
		<p>
			<label for="<?php echo $this->get_field_id( 'map_size' ); ?>"><?php _e('Map Size: ', 'four2map'); ?></label><input id="<?php echo $this->get_field_id( 'map_size' ); ?>" name="<?php echo $this->get_field_name( 'map_size' ); ?>" value="<?php echo $instance['map_size']; ?>" style="width:30px;" /><?php _e(' pixels', 'four2map'); ?>	
		</p>
		
		<!-- Map Zoom: Text Input -->
		<p>
			<label for="<?php echo $this->get_field_id( 'map_zoom' ); ?>"><?php _e('Map Zoom: ', 'four2map'); ?></label><input id="<?php echo $this->get_field_id( 'map_zoom' ); ?>" name="<?php echo $this->get_field_name( 'map_zoom' ); ?>" value="<?php echo $instance['map_zoom']; ?>" style="width:30px;" /><?php _e(' (0 min, 21 max)', 'four2map'); ?>	
		</p>

		<!-- Show Label Checkbox -->
		<p>
			<input class="checkbox" type="checkbox" <?php checked( (bool) $instance['label_venue'], true ); ?> id="<?php echo $this->get_field_id( 'label_venue' ); ?>" name="<?php echo $this->get_field_name( 'label_venue' ); ?>" /> 
			<label for="<?php echo $this->get_field_id( 'label_venue' ); ?>"><?php _e('Show the venue name?', 'four2map'); ?></label>
		</p>
		
		<!-- Show Comment Checkbox -->
		<p>
			<input class="checkbox" type="checkbox" <?php checked( (bool) $instance['show_comment'], true ); ?> id="<?php echo $this->get_field_id( 'show_comment' ); ?>" name="<?php echo $this->get_field_name( 'show_comment' ); ?>" /> 
			<label for="<?php echo $this->get_field_id( 'show_comment' ); ?>"><?php _e('Show your last comment?', 'four2map'); ?></label>
		</p>

	<?php
	}
}

/* 
 * Foursquare-php location fetcher class
 * Connect to foursquare and retrieve your last location to display it along with a google map on your page
 * Author: Elie Bursztein (fourlocfetcher@elie.im)
 * URL:http://code.google.com/p/foursquare-php/
 * Version: 0.1
 * License: GPL
 */

class fourSquare {

	private $user = "";
	private $pass = "";
	private $url = "http://api.foursquare.com/v1/user.json"; //foursquare api call
	
	public $userId = "";
	public $date = "";
	public $venueId = "";
	public $venueName = "";
	public $venueCat = "";
	public $venueType = "";
	public $venueIcon = "http://foursquare.com/img/categories/question.png";
	public $comment = "";
	public $address = "";
	public $city = "";
	public $state = "";
	public $geolong = "";
	public $geolat = "";
  
	/*
	* @param $user foursquare username or email
	* @param $pass foursquare password
	*/
	function __construct($user, $pass) {
		   $this->user = $user;
		   $this->pass = $pass;

		   //fetching data
		   $ch = curl_init();
		   curl_setopt($ch, CURLOPT_URL, $this->url);
		   curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		   curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC) ;
		   curl_setopt($ch, CURLOPT_USERPWD, $user.":".$pass);	
		   curl_setopt($ch, CURLOPT_USERAGENT, "fetcher " . time()); //prevent rate limit
		   curl_setopt($ch, CURLOPT_SSLVERSION,3);
		   $data = curl_exec($ch);
		   curl_close($ch);
		   
		   //decoding data
		   $fd = json_decode($data);

		   if (isset($fd->{"unauthorized"})) {
				echo "There was a problem logging in to Foursquare. Oh noes!";
				return;
		   }

		   //parsing
		   try {
			   $this->userId = $fd->{"user"}->{"id"};
			   $this->date = $fd->{"user"}->{"checkin"}->{"created"};
			   $this->venueId = $fd->{"user"}->{"checkin"}->{"venue"}->{"id"};
			   $this->venueName = $fd->{"user"}->{"checkin"}->{"venue"}->{"name"};
			   
			   if (isset($fd->{"user"}->{"checkin"}->{"venue"}->{"primarycategory"}->{"fullpathname"})) {
					$this->venueCat =  $fd->{"user"}->{"checkin"}->{"venue"}->{"primarycategory"}->{"fullpathname"};
					$this->venueCat = str_replace(':', '/', $this->venueCat);
			   }
			   
			   if (isset($fd->{"user"}->{"checkin"}->{"venue"}->{"primarycategory"}->{"nodename"})) {
					$this->venueType = $fd->{"user"}->{"checkin"}->{"venue"}->{"primarycategory"}->{"nodename"};
					$this->venueIcon = $fd->{"user"}->{"checkin"}->{"venue"}->{"primarycategory"}->{"iconurl"};
			   }
			   
			   if (isset($fd->{"user"}->{"checkin"}->{"shout"})) {
				   $this->comment = $fd->{"user"}->{"checkin"}->{"shout"};
			   }
	
			   if (isset($fd->{"user"}->{"checkin"}->{"venue"}->{"address"})) {
					$this->address =  $fd->{"user"}->{"checkin"}->{"venue"}->{"address"};
					$this->city = $fd->{"user"}->{"checkin"}->{"venue"}->{"city"};
					$this->state = $fd->{"user"}->{"checkin"}->{"venue"}->{"state"};
					$this->geolat = $fd->{"user"}->{"checkin"}->{"venue"}->{"geolat"};
					$this->geolong = $fd->{"user"}->{"checkin"}->{"venue"}->{"geolong"};
			   }
		   } 
		   catch(Exception $e) {
			   
		   }

	}

	  /*
	  * get the google map img url corresponding to the foursquare location
	  * @param $width: map width
	  * @param $height: map height
	  * @param $mobile: is this a map for a mobile page? - default false
	  * @param $maptype: what type of map to render. possible choice : roadmap, satellite, hybrid, and terrain - default "roadmap"
	  * @param $zoom: level of zoom in the map - default 12
	  * @param $markerText: text on the marker - default "" (can only be a single uppercase letter) put "" for a .
	  * @param $markerText: color of the marker - default blue
	  */

	public function getMapUrl($width, $height, $zoom = 12, $iconUrl = "", $markerColor = "red", $mobile = FALSE, $maptype = "roadmap") {

		$mapUrl      = "http://maps.google.com/maps/api/staticmap?";      	// static map url
		$mapUrl     .= "sensor=true";                                     	// we are using the GPS through foursquare so it is a sensor map.
		$mapUrl     .= "&center=". $this->geolat . ",". $this->geolong ;   	// position returned by foursquare
		$mapUrl     .= "&maptype=" . $maptype;
		$mapUrl     .= "&size=". $width . "x" . $height;
		$mapUrl     .= "&zoom=" . $zoom;
		$markerText = strtoupper(substr($markerText, 0, 1));
		if ($iconUrl == "") {
			$mapUrl .= "&markers=color:".$markerColor."|label:".$markerText;
		}
		else {
			$mapUrl .= "&markers=icon:".$iconUrl;
		}
		$mapUrl .= "|".$this->geolat.",".$this->geolong . "|";

		return $mapUrl;

	}

}

?>